/**
 *  An object representing an author or committer, containing their name and email address.
 */
export interface Person {
    name: string;
    email: string;
}
/**
 *  An object containing relative paths to the files added, deleted, or modified by each commit.
 */
export interface GitDiff {
    added: Set<string>;
    deleted: Set<string>;
    modified: Set<string>;
    touched: Set<string>;
}
/**
 *  An object containing the metadata for a single git commit.
 */
export interface Commit {
    fullHash: string;
    partialHash: string;
    author: Person;
    authorTime: Date;
    committer: Person;
    commitTime: Date;
    subject: string;
    body: string;
    diff?: GitDiff;
}
/**
 *  Asynchronously fetches the metadata of all commits within a particular reference range.
 *  @param dir The path to the root directory of a git repository.
 *  @param startRef A reference string, such as a commit hash or tag name, which designates the
 *    beginning of the range (exclusive.) If not defined, all ancestors of endRef will be listed.
 *  @param endRef A reference string, such as a commit hash or tag name, which designates the end
 *    of the range (inclusive.) If not defined, endRef will be assumed to be 'HEAD'.
 *  @param includeDiff An optional boolean which, if set to true, will cause information on the
 *    files touched by each commit to also be collected. This functionality is disabled by default
 *    for performance reasons.
 *  @return A Promise for an array of objects containing the metadata of each commit in the range.
 */
export declare function gitLog(dir?: string, startRef?: string, endRef?: string, includeDiff?: boolean): Promise<Commit[]>;
/**
 *  Synchronously fetches the metadata of all commits within a particular reference range.
 *  @param dir The path to the root directory of a git repository.
 *  @param startRef A reference string, such as a commit hash or tag name, which designates the
 *    beginning of the range (exclusive.) If not defined, all ancestors of endRef will be listed.
 *  @param endRef A reference string, such as a commit hash or tag name, which designates the end
 *    of the range (inclusive.) If not defined, endRef will be assumed to be 'HEAD'.
 *  @param includeDiff An optional boolean which, if set to true, will cause information on the
 *    files touched by each commit to also be collected. This functionality is disabled by default
 *    for performance reasons.
 *  @return An array of objects containing the metadata of each commit in the range.
 */
export declare function gitLogSync(dir?: string, startRef?: string, endRef?: string, includeDiff?: boolean): Commit[];
