"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const gitLogArgs = (range) => [
    'log',
    range,
    '--format=\x1F%H\x1F%h\x1F%an\x1F%ae\x1F%at\x1F%cn\x1F%ce\x1F%ct\x1F%s\x1F%b\x1F\x1E',
];
const gitDiffArgs = (commit) => [
    'diff-tree',
    '--no-commit-id',
    '--name-status',
    '-r',
    commit.fullHash,
];
/**
 *  Asynchronously fetches the metadata of all commits within a particular reference range.
 *  @param dir The path to the root directory of a git repository.
 *  @param startRef A reference string, such as a commit hash or tag name, which designates the
 *    beginning of the range (exclusive.) If not defined, all ancestors of endRef will be listed.
 *  @param endRef A reference string, such as a commit hash or tag name, which designates the end
 *    of the range (inclusive.) If not defined, endRef will be assumed to be 'HEAD'.
 *  @param includeDiff An optional boolean which, if set to true, will cause information on the
 *    files touched by each commit to also be collected. This functionality is disabled by default
 *    for performance reasons.
 *  @return A Promise for an array of objects containing the metadata of each commit in the range.
 */
function gitLog(dir = process.cwd(), startRef, endRef, includeDiff = false) {
    return __awaiter(this, void 0, void 0, function* () {
        const rangeString = getRangeString(startRef, endRef);
        const log = yield wrapGitProcess(gitLogArgs(rangeString), dir).then(parseLog);
        if (includeDiff) {
            const applyDiff = (commit) => wrapGitProcess(gitDiffArgs(commit), dir)
                .then(parseDiff)
                .then(diff => Object.assign(commit, { diff }));
            return Promise.all(log.map(applyDiff));
        }
        return log;
    });
}
exports.gitLog = gitLog;
/**
 *  Synchronously fetches the metadata of all commits within a particular reference range.
 *  @param dir The path to the root directory of a git repository.
 *  @param startRef A reference string, such as a commit hash or tag name, which designates the
 *    beginning of the range (exclusive.) If not defined, all ancestors of endRef will be listed.
 *  @param endRef A reference string, such as a commit hash or tag name, which designates the end
 *    of the range (inclusive.) If not defined, endRef will be assumed to be 'HEAD'.
 *  @param includeDiff An optional boolean which, if set to true, will cause information on the
 *    files touched by each commit to also be collected. This functionality is disabled by default
 *    for performance reasons.
 *  @return An array of objects containing the metadata of each commit in the range.
 */
// tslint:disable-next-line max-line-length
function gitLogSync(dir = process.cwd(), startRef, endRef, includeDiff = false) {
    const rangeString = getRangeString(startRef, endRef);
    const log = parseLog(wrapGitProcessSync(gitLogArgs(rangeString), dir));
    if (includeDiff) {
        for (const commit of log) {
            const diff = parseDiff(wrapGitProcessSync(gitDiffArgs(commit), dir));
            Object.assign(commit, { diff });
        }
    }
    return log;
}
exports.gitLogSync = gitLogSync;
function getRangeString(startRef, endRef = 'HEAD') {
    if (startRef !== undefined) {
        return `${startRef}..${endRef}`;
    }
    return endRef;
}
function wrapGitProcess(args, cwd) {
    return new Promise((resolve, reject) => {
        const child = child_process_1.spawn('git', args, { cwd });
        let stdout = new Buffer('');
        let stderr = new Buffer('');
        child.stdout.on('data', data => stdout = Buffer.concat([stdout, data]));
        child.stderr.on('data', data => stderr = Buffer.concat([stderr, data]));
        child.once('close', (code) => {
            if (code !== 0 || stderr.toString() !== '') {
                reject(getErrorWithCode(code, stderr.toString()));
            }
            else {
                resolve(stdout.toString());
            }
        });
    });
}
function wrapGitProcessSync(args, cwd) {
    const child = child_process_1.spawnSync('git', args, { cwd });
    if (child.status !== 0) {
        throw getErrorWithCode(child.status, child.stderr.toString());
    }
    return child.stdout.toString();
}
function parseLog(rawData) {
    const commits = [];
    const commitData = rawData.split('\x1E');
    for (let i = 0; i < commitData.length - 1; i += 1) {
        const entries = commitData[i].split('\x1F');
        commits[i] = {
            fullHash: entries[1],
            partialHash: entries[2],
            author: {
                name: entries[3],
                email: entries[4],
            },
            authorTime: new Date(parseInt(entries[5], 10) * 1000),
            committer: {
                name: entries[6],
                email: entries[7],
            },
            commitTime: new Date(parseInt(entries[8], 10) * 1000),
            subject: entries[9],
            body: entries[10],
        };
    }
    return commits;
}
function parseDiff(rawData) {
    const files = rawData.split('\n').slice(0, -1).map(s => s.split('\t'));
    const diff = {
        added: new Set(),
        deleted: new Set(),
        modified: new Set(),
        touched: new Set(),
    };
    for (const [mode, path] of files) {
        diff.touched.add(path);
        switch (mode) {
            case 'A':
                diff.added.add(path);
                continue;
            case 'D':
                diff.deleted.add(path);
                continue;
            case 'M':
                diff.modified.add(path);
                continue;
        }
    }
    return diff;
}
function getErrorWithCode(code, message) {
    const err = new Error(message);
    err.code = code;
    return err;
}
